import logging
import config

class GPIOInterface:

    def __init__(self):
        self.GPIO = None
        self.logger = logging.getLogger(self.__class__.__name__)
        if config.get_settings().get('gpio').get('enabled'):
            try:
                import RPi.GPIO as GPIO
            except ImportError:
                print("Error importing RPi.GPIO.")
            else:
                GPIO.setmode(GPIO.BOARD)
                GPIO.setwarnings(False)
                self.active_buttons = []
                self.cancel_callbacks = []
                self.buttons = []
                self.GPIO = GPIO
                buttons = config.get_settings()['gpio'].get('buttons', {})
                for button in buttons:
                    action_name = button
                    pin_number = buttons[button]
                    self.buttons.append((pin_number, action_name))
                    GPIO.setup(pin_number, GPIO.IN, pull_up_down=GPIO.PUD_UP)
                    GPIO.add_event_detect(pin_number, GPIO.RISING, bouncetime=200)
                    self.logger.info("GPIO event registered: pin%d=%s" % (pin_number, action_name))

    def check_events(self, printer_interfaces):
        if self.GPIO:
            for pin_num, action_name in self.buttons:
                if self.GPIO.event_detected(pin_num):
                    for pi in printer_interfaces:
                        self.run_method(pi, action_name)

    def run_method(self, pi, method_name):
        method = getattr(pi, method_name, None)
        if method:
            try:
                method()
            except Exception:
                self.logger.exception("Exception in GPIO handler")
        else:
            self.logger.error("GPIO event error: no method %s" % method_name)

    def close(self):
        if self.GPIO:
            self.GPIO.cleanup()

